# -*- coding: utf-8 -*-
# Copyright (c) 2016  Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Written by Ralph Bean <rbean@redhat.com>

""" Handlers for koji component build events on the message bus. """

import logging

import module_build_service.builder

import koji

from module_build_service import models, log, messaging

logging.basicConfig(level=logging.DEBUG)


def _finalize(config, db_session, msg, state):
    """ Called whenever a koji build completes or fails. """

    # First, find our ModuleBuild associated with this component, if any.
    component_build = models.ComponentBuild.from_component_event(db_session, msg)
    try:
        nvr = "{}-{}-{}".format(msg.build_name, msg.build_version, msg.build_release)
    except KeyError:
        nvr = None

    if not component_build:
        log.debug("We have no record of %s" % nvr)
        return

    log.info("Saw relevant component build of %r from %r." % (nvr, msg.msg_id))

    if msg.state_reason:
        state_reason = msg.state_reason
    elif state != koji.BUILD_STATES["COMPLETE"]:
        state_reason = "Failed to build artifact %s in Koji" % (msg.build_name)
    else:
        state_reason = ""

    # Mark the state in the db.
    component_build.state = state
    component_build.nvr = nvr
    component_build.state_reason = state_reason
    db_session.commit()

    parent = component_build.module_build

    # If the macro build failed, then the module is doomed.
    if component_build.package == "module-build-macros" and state != koji.BUILD_STATES["COMPLETE"]:
        parent.transition(
            db_session,
            config,
            state=models.BUILD_STATES["failed"],
            state_reason=state_reason,
            failure_type="user",
        )
        db_session.commit()
        return

    further_work = []

    parent_current_batch = parent.current_batch()

    # If there are no other components still building in a batch,
    # we can tag all successfully built components in the batch.
    unbuilt_components_in_batch = [
        c for c in parent_current_batch
        if c.is_waiting_for_build or c.is_building
    ]
    if not unbuilt_components_in_batch:
        failed_components_in_batch = [c for c in parent_current_batch if c.is_unsuccessful]
        built_components_in_batch = [c for c in parent_current_batch if c.is_completed]

        builder = module_build_service.builder.GenericBuilder.create_from_module(
            db_session, parent, config
        )

        if failed_components_in_batch:
            log.info(
                "Batch done, but not tagging because of failed component builds. Will "
                'transition the module to "failed"'
            )
            state_reason = "Component(s) {} failed to build.".format(
                ", ".join(c.package for c in failed_components_in_batch))
            parent.transition(
                db_session,
                config,
                state=models.BUILD_STATES["failed"],
                state_reason=state_reason,
                failure_type="user",
            )
            db_session.commit()
            return []
        elif not built_components_in_batch:
            # If there are no successfully built components in a batch, there is nothing to tag.
            # The repository won't be regenerated in this case and therefore we generate fake repo
            # change message here.
            log.info("Batch done. No component to tag")
            further_work += [
                messaging.KojiRepoChange(
                    "components::_finalize: fake msg", builder.module_build_tag["name"])
            ]
        else:
            built_component_nvrs_in_batch = [c.nvr for c in built_components_in_batch]
            # tag && add to srpm-build group if neccessary
            log.info(
                "Batch done.  Tagging %i component(s) in the build tag."
                % len(built_component_nvrs_in_batch)
            )
            log.debug("%r" % built_component_nvrs_in_batch)
            # TODO: install=component_build.build_time_only works here because module-build-macros
            # is alone in its batch and the only component with build_time_only set. All other
            # batches will have install=False. If we expand to have batches with a mix of
            # build_time_only and not components, then this logic will need to change.
            builder.buildroot_add_artifacts(
                built_component_nvrs_in_batch, install=component_build.build_time_only)

            # Do not tag packages which only belong to the build tag to the dest tag
            component_nvrs_to_tag_in_dest = [
                c.nvr for c in built_components_in_batch
                if c.build_time_only is False
            ]
            log.info(
                "Tagging %i component(s) in the dest tag." % len(component_nvrs_to_tag_in_dest))
            if component_nvrs_to_tag_in_dest:
                builder.tag_artifacts(component_nvrs_to_tag_in_dest)

        db_session.commit()

    elif any(not c.is_building for c in unbuilt_components_in_batch):
        # We are not in the middle of the batch building and
        # we have some unbuilt components in this batch. We might hit the
        # concurrent builds threshold in previous call of continue_batch_build
        # done in repos.py:done(...), but because we have just finished one
        # build, try to call continue_batch_build again so in case we hit the
        # threshold previously, we will submit another build from this batch.
        builder = module_build_service.builder.GenericBuilder.create_from_module(
            db_session, parent, config)
        further_work += module_build_service.utils.continue_batch_build(
            config, parent, db_session, builder)

    return further_work


def complete(config, db_session, msg):
    return _finalize(config, db_session, msg, state=koji.BUILD_STATES["COMPLETE"])


def failed(config, db_session, msg):
    return _finalize(config, db_session, msg, state=koji.BUILD_STATES["FAILED"])


def canceled(config, db_session, msg):
    return _finalize(config, db_session, msg, state=koji.BUILD_STATES["CANCELED"])



# Copyright (c) 2019  Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import pytest
from mock import patch

from module_build_service import app, models
from module_build_service.manage import manager_wrapper, retire
from module_build_service.models import BUILD_STATES, ModuleBuild
from module_build_service.utils.general import deps_to_dict
from tests import clean_database, staged_data_filename


@pytest.mark.usefixtures("model_tests_init_data")
class TestMBSManage:

    @pytest.mark.parametrize(
        ("identifier", "is_valid"),
        (
            ("", False),
            ("spam", False),
            ("spam:bacon", True),
            ("spam:bacon:eggs", True),
            ("spam:bacon:eggs:ham", True),
            ("spam:bacon:eggs:ham:sausage", False),
        ),
    )
    def test_retire_identifier_validation(self, identifier, is_valid):
        if is_valid:
            retire(identifier)
        else:
            with pytest.raises(ValueError):
                retire(identifier)

    @pytest.mark.parametrize(
        ("overrides", "identifier", "changed_count"),
        (
            ({"name": "pickme"}, "pickme:eggs", 1),
            ({"stream": "pickme"}, "spam:pickme", 1),
            ({"version": "pickme"}, "spam:eggs:pickme", 1),
            ({"context": "pickme"}, "spam:eggs:ham:pickme", 1),
            ({}, "spam:eggs", 3),
            ({"version": "pickme"}, "spam:eggs", 3),
            ({"context": "pickme"}, "spam:eggs:ham", 3),
        ),
    )
    @patch("module_build_service.manage.prompt_bool")
    def test_retire_build(self, prompt_bool, overrides, identifier, changed_count, db_session):
        prompt_bool.return_value = True

        module_builds = db_session.query(ModuleBuild).filter_by(state=BUILD_STATES["ready"]).all()
        # Verify our assumption of the amount of ModuleBuilds in database
        assert len(module_builds) == 3

        for x, build in enumerate(module_builds):
            build.name = "spam"
            build.stream = "eggs"
            build.version = "ham"
            build.context = str(x)

        for attr, value in overrides.items():
            setattr(module_builds[0], attr, value)

        db_session.commit()

        retire(identifier)
        retired_module_builds = (
            db_session.query(ModuleBuild).filter_by(state=BUILD_STATES["garbage"]).all()
        )

        assert len(retired_module_builds) == changed_count
        for x in range(changed_count):
            assert retired_module_builds[x].id == module_builds[x].id
            assert retired_module_builds[x].state == BUILD_STATES["garbage"]

    @pytest.mark.parametrize(
        ("confirm_prompt", "confirm_arg", "confirm_expected"),
        (
            (True, False, True),
            (True, True, True),
            (False, False, False),
            (False, True, True)
        ),
    )
    @patch("module_build_service.manage.prompt_bool")
    def test_retire_build_confirm_prompt(
        self, prompt_bool, confirm_prompt, confirm_arg, confirm_expected, db_session
    ):
        prompt_bool.return_value = confirm_prompt

        module_builds = db_session.query(ModuleBuild).filter_by(state=BUILD_STATES["ready"]).all()
        # Verify our assumption of the amount of ModuleBuilds in database
        assert len(module_builds) == 3

        for x, build in enumerate(module_builds):
            build.name = "spam"
            build.stream = "eggs"

        db_session.commit()

        retire("spam:eggs", confirm_arg)
        retired_module_builds = (
            db_session.query(ModuleBuild).filter_by(state=BUILD_STATES["garbage"]).all()
        )

        expected_changed_count = 3 if confirm_expected else 0
        assert len(retired_module_builds) == expected_changed_count


class TestCommandBuildModuleLocally:
    """Test mbs-manager subcommand build_module_locally"""

    def setup_method(self, test_method):
        clean_database()

        # Do not allow flask_script exits by itself because we have to assert
        # something after the command finishes.
        self.sys_exit_patcher = patch("sys.exit")
        self.mock_sys_exit = self.sys_exit_patcher.start()

        # The consumer is not required to run actually, so it does not make
        # sense to publish message after creating a module build.
        self.publish_patcher = patch("module_build_service.messaging.publish")
        self.mock_publish = self.publish_patcher.start()

        # Don't allow conf.set_item call to modify conf actually inside command
        self.set_item_patcher = patch("module_build_service.manage.conf.set_item")
        self.mock_set_item = self.set_item_patcher.start()

        # Avoid to create the local sqlite database for the command, which is
        # useless for running tests here.
        self.create_all_patcher = patch("module_build_service.manage.db.create_all")
        self.mock_create_all = self.create_all_patcher.start()

    def teardown_method(self, test_method):
        self.create_all_patcher.stop()
        self.mock_set_item.stop()
        self.publish_patcher.stop()
        self.sys_exit_patcher.stop()

    def _run_manager_wrapper(self, cli_cmd):
        # build_module_locally changes database uri to a local SQLite database file.
        # Restore the uri to original one in order to not impact the database
        # session in subsequent tests.
        original_db_uri = app.config["SQLALCHEMY_DATABASE_URI"]
        try:
            with patch("sys.argv", new=cli_cmd):
                manager_wrapper()
        finally:
            app.config["SQLALCHEMY_DATABASE_URI"] = original_db_uri

    @patch("module_build_service.scheduler.main")
    def test_set_stream(self, main, db_session):
        cli_cmd = [
            "mbs-manager", "build_module_locally",
            "--set-stream", "platform:f28",
            "--file", staged_data_filename("testmodule-local-build.yaml")
        ]

        self._run_manager_wrapper(cli_cmd)

        # Since module_build_service.scheduler.main is mocked, MBS does not
        # really build the testmodule for this test. Following lines assert the
        # fact:
        # Module testmodule-local-build is expanded and stored into database,
        # and this build has buildrequires platform:f28 and requires
        # platform:f28.
        # Please note that, the f28 is specified from command line option
        # --set-stream, which is the point this test tests.

        builds = db_session.query(models.ModuleBuild).filter_by(
            name="testmodule-local-build").all()
        assert 1 == len(builds)

        testmodule_build = builds[0]
        mmd_deps = testmodule_build.mmd().get_dependencies()

        deps_dict = deps_to_dict(mmd_deps[0], "buildtime")
        assert ["f28"] == deps_dict["platform"]
        deps_dict = deps_to_dict(mmd_deps[0], "runtime")
        assert ["f28"] == deps_dict["platform"]

    @patch("module_build_service.manage.logging")
    def test_ambiguous_stream(self, logging):
        cli_cmd = [
            "mbs-manager", "build_module_locally",
            "--file", staged_data_filename("testmodule-local-build.yaml")
        ]

        self._run_manager_wrapper(cli_cmd)

        args, _ = logging.error.call_args_list[0]
        assert "There are multiple streams to choose from for module platform." == args[0]
        args, _ = logging.error.call_args_list[1]
        assert "Use '-s module_name:module_stream' to choose the stream" == args[0]

    def test_module_build_failed(self, db_session):
        cli_cmd = [
            "mbs-manager", "build_module_locally",
            "--set-stream", "platform:f28",
            "--file", staged_data_filename("testmodule-local-build.yaml")
        ]

        def main_side_effect(initial_messages, stop_condition):
            build = db_session.query(models.ModuleBuild).filter(
                models.ModuleBuild.name == "testmodule-local-build"
            ).first()
            build.state = models.BUILD_STATES["failed"]
            db_session.commit()

        # We don't run consumer actually, but it could be patched to mark some
        # module build failed for test purpose.

        with patch("module_build_service.scheduler.main",
                   side_effect=main_side_effect):
            with pytest.raises(RuntimeError, match="Module build failed"):
                self._run_manager_wrapper(cli_cmd)



# Copyright (c) 2016  Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Written by Ralph Bean <rbean@redhat.com>

import mock

import module_build_service.messaging
import module_build_service.scheduler.handlers.repos
import module_build_service.models
from module_build_service.models import ComponentBuild
from tests import conf, scheduler_init_data


class TestRepoDone:

    @mock.patch("module_build_service.models.ModuleBuild.from_repo_done_event")
    def test_no_match(self, from_repo_done_event, db_session):
        """ Test that when a repo msg hits us and we have no match,
        that we do nothing gracefully.
        """
        scheduler_init_data(db_session)
        from_repo_done_event.return_value = None
        msg = module_build_service.messaging.KojiRepoChange(
            "no matches for this...", "2016-some-nonexistent-build")
        module_build_service.scheduler.handlers.repos.done(
            config=conf, db_session=db_session, msg=msg)

    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.recover_orphaned_artifact",
        return_value=[],
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.get_average_build_time",
        return_value=0.0,
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.list_tasks_for_components",
        return_value=[],
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_ready",
        return_value=True,
    )
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.get_session")
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.build")
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_connect"
    )
    def test_a_single_match(
        self, connect, build_fn, get_session, ready, list_tasks_fn, mock_gabt, mock_uea, db_session
    ):
        """ Test that when a repo msg hits us and we have a single match.
        """
        scheduler_init_data(db_session)
        get_session.return_value = mock.Mock(), "development"
        build_fn.return_value = 1234, 1, "", None

        msg = module_build_service.messaging.KojiRepoChange(
            "some_msg_id", "module-testmodule-master-20170109091357-7c29193d-build")
        module_build_service.scheduler.handlers.repos.done(
            config=conf, db_session=db_session, msg=msg)
        build_fn.assert_called_once_with(
            artifact_name="tangerine",
            source=(
                "https://src.fedoraproject.org/rpms/tangerine?"
                "#fbed359411a1baa08d4a88e0d12d426fbf8f602c"
            ),
        )

    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.finalize")
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.recover_orphaned_artifact",
        return_value=[],
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.get_average_build_time",
        return_value=0.0,
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.list_tasks_for_components",
        return_value=[],
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_ready",
        return_value=True,
    )
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.get_session")
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.build")
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_connect"
    )
    def test_a_single_match_finalize(
        self, connect, build_fn, get_session, ready, list_tasks_fn, mock_gabt, mock_uea, finalizer,
        db_session
    ):
        """ Test that when a repo msg hits us and we have a single match.
        """
        scheduler_init_data(db_session, tangerine_state=1)
        get_session.return_value = mock.Mock(), "development"
        build_fn.return_value = 1234, 1, "", None

        # Ensure the time_completed is None, so we can test it is set to
        # some date once the build is finalized.
        module_build = module_build_service.models.ModuleBuild.get_by_id(db_session, 2)
        module_build.time_completed = None
        db_session.commit()

        def mocked_finalizer(succeeded=None):
            # Check that the time_completed is set in the time when
            # finalizer is called.
            assert succeeded is True
            module_build = module_build_service.models.ModuleBuild.get_by_id(db_session, 2)
            assert module_build.time_completed is not None

        finalizer.side_effect = mocked_finalizer

        msg = module_build_service.messaging.KojiRepoChange(
            "some_msg_id", "module-testmodule-master-20170109091357-7c29193d-build")
        module_build_service.scheduler.handlers.repos.done(
            config=conf, db_session=db_session, msg=msg)

        finalizer.assert_called_once()

    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.recover_orphaned_artifact",
        return_value=[],
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.get_average_build_time",
        return_value=0.0,
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.list_tasks_for_components",
        return_value=[],
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_ready",
        return_value=True,
    )
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.get_session")
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.build")
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_connect"
    )
    def test_a_single_match_build_fail(
        self, connect, build_fn, config, ready, list_tasks_fn, mock_gabt, mock_uea, db_session
    ):
        """ Test that when a KojiModuleBuilder.build fails, the build is
        marked as failed with proper state_reason.
        """
        scheduler_init_data(db_session)
        config.return_value = mock.Mock(), "development"
        build_fn.return_value = None, 4, "Failed to submit artifact tangerine to Koji", None

        msg = module_build_service.messaging.KojiRepoChange(
            "some_msg_id", "module-testmodule-master-20170109091357-7c29193d-build")
        module_build_service.scheduler.handlers.repos.done(
            config=conf, db_session=db_session, msg=msg)
        build_fn.assert_called_once_with(
            artifact_name="tangerine",
            source=(
                "https://src.fedoraproject.org/rpms/tangerine?"
                "#fbed359411a1baa08d4a88e0d12d426fbf8f602c"
            ),
        )
        component_build = db_session.query(
            module_build_service.models.ComponentBuild
        ).filter_by(package="tangerine").one()
        assert component_build.state_reason == "Failed to submit artifact tangerine to Koji"

    @mock.patch("module_build_service.scheduler.handlers.repos.log.info")
    def test_erroneous_regen_repo_received(self, mock_log_info, db_session):
        """ Test that when an unexpected KojiRepoRegen message is received, the module doesn't
        complete or go to the next build batch.
        """
        scheduler_init_data(db_session, 1)

        component_build = db_session.query(ComponentBuild).filter_by(package="tangerine").one()
        component_build.tagged = False
        db_session.commit()

        msg = module_build_service.messaging.KojiRepoChange(
            "some_msg_id", "module-testmodule-master-20170109091357-7c29193d-build")

        module_build_service.scheduler.handlers.repos.done(
            config=conf, db_session=db_session, msg=msg)

        mock_log_info.assert_called_with(
            "Ignoring repo regen, because not all components are tagged."
        )
        module_build = module_build_service.models.ModuleBuild.get_by_id(db_session, 2)
        # Make sure the module build didn't transition since all the components weren't tagged
        assert module_build.state == module_build_service.models.BUILD_STATES["build"]

    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder."
        "KojiModuleBuilder.list_tasks_for_components",
        return_value=[],
    )
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_ready",
        return_value=True,
    )
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.get_session")
    @mock.patch("module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.build")
    @mock.patch(
        "module_build_service.builder.KojiModuleBuilder.KojiModuleBuilder.buildroot_connect"
    )
    @mock.patch(
        "module_build_service.builder.GenericBuilder.default_buildroot_groups",
        return_value={"build": [], "srpm-build": []},
    )
    def test_failed_component_build(
        self, dbg, connect, build_fn, config, ready, list_tasks_fn, db_session
    ):
        """ Test that when a KojiModuleBuilder.build fails, the build is
        marked as failed with proper state_reason.
        """
        scheduler_init_data(db_session, 3)
        config.return_value = mock.Mock(), "development"
        build_fn.return_value = None, 4, "Failed to submit artifact x to Koji", None

        msg = module_build_service.messaging.KojiRepoChange(
            "some_msg_id", "module-testmodule-master-20170109091357-7c29193d-build")
        module_build_service.scheduler.handlers.repos.done(
            config=conf, db_session=db_session, msg=msg)
        module_build = module_build_service.models.ModuleBuild.get_by_id(db_session, 2)

        assert module_build.state == module_build_service.models.BUILD_STATES["failed"]
